import { useQuerybuilderGraph } from '@graphpolaris/shared/lib/data-access';
import React, { useMemo, useRef, useState } from 'react';
import { Handle, Position, useUpdateNodeInternals } from 'reactflow';
import { NodeAttribute, SchemaReactflowEntityNode, toHandleId } from '../../../model';
import { PillDropdown } from '../../pilldropdown/PillDropdown';
import { EntityPill } from '@graphpolaris/shared/lib/components';

/**
 * Component to render an entity flow element
 * @param {NodeProps} param0 The data of an entity flow element.
 */
export const QueryEntityPill = React.memo((node: SchemaReactflowEntityNode) => {
  const updateNodeInternals = useUpdateNodeInternals();
  const ref = useRef<HTMLDivElement | null>(null);
  const dropdownActive = useRef(false);

  const data = node.data;
  if (!data.leftRelationHandleId) throw new Error('EntityFlowElement: data.leftRelationHandleId is undefined');
  if (!data.rightRelationHandleId) throw new Error('EntityFlowElement: data.rightRelationHandleId is undefined');

  const graph = useQuerybuilderGraph();
  const attributeEdges = useMemo(
    () => graph.edges.filter((edge) => edge.source === node.id && !!edge?.attributes?.sourceHandleData.attributeType),
    [graph],
  );

  const [hovered, setHovered] = useState(false);
  const [dragging, setDragging] = useState(false);
  const [handleBeingDragged, setHandleBeingDragged] = useState(-1);

  const onMouseEnter = (event: React.MouseEvent) => {
    if (!hovered) setHovered(true);
    ref.current?.addEventListener('mousedown', onMouseDown, true);
    setTimeout(() => {
      updateNodeInternals(node.id);
    }, 100);
  };

  const onMouseLeave = (event: React.MouseEvent) => {
    ref.current?.removeEventListener('mousedown', onMouseDown, true);
    if (hovered) setHovered(false);
    setTimeout(() => {
      updateNodeInternals(node.id);
    }, 100);
  };

  const onHandleMouseDown = (attribute: NodeAttribute, i: number, event: React.MouseEvent) => {
    setHandleBeingDragged(i);
    window.addEventListener('mouseup', onHandleMouseUp, true);
  };

  const onHandleMouseUp = () => {
    setHandleBeingDragged(-1);
    window.removeEventListener('mouseup', onHandleMouseUp, true);
  };

  const onMouseDown = React.useCallback(() => {
    window.addEventListener('mouseup', onMouseUp, true);
    setDragging(true);
  }, []);

  const onMouseUp = () => {
    setDragging(false);
    setHovered(true);
    window.removeEventListener('mouseup', onMouseUp, true);
  };

  const onConnect = (params: any) => {
    console.log('EntityPill onConnect', params);
  };

  const onMouseEnterDropdown = (event: React.MouseEvent) => {
    ref.current?.removeEventListener('mousedown', onMouseDown, true);
  };

  const onMouseLeaveDropdown = (event: React.MouseEvent) => {
    ref.current?.addEventListener('mousedown', onMouseDown, true);
  };

  return (
    <div className="w-fit h-fit nowheel" onMouseEnter={onMouseEnter} onMouseLeave={onMouseLeave} ref={ref} id="asd">
      <EntityPill
        title={data.name || ''}
        withHandles="horizontal"
        handleLeft={
          <Handle
            id={toHandleId(data.leftRelationHandleId)}
            position={Position.Left}
            className={'!rounded-none !bg-transparent !w-full !h-full !border-0 !right-0 !left-0'}
            type="target"
          ></Handle>
        }
        handleRight={
          <Handle
            id={toHandleId(data.rightRelationHandleId)}
            // id="entitySourceRight"
            position={Position.Right}
            className={'!rounded-none !bg-transparent !w-full !h-full !border-0 !right-0 !left-0'}
            type="source"
          ></Handle>
        }
      >
        {data?.attributes && (
          <PillDropdown
            onMouseEnterDropdown={onMouseEnterDropdown}
            onMouseLeaveDropdown={onMouseLeaveDropdown}
            node={node}
            attributes={data.attributes}
            attributeEdges={attributeEdges.map((edge) => edge?.attributes)}
            hovered={hovered && !dragging}
            handleBeingDraggedIdx={handleBeingDragged}
            onHandleMouseDown={onHandleMouseDown}
          />
        )}
      </EntityPill>
    </div>
  );
});

QueryEntityPill.displayName = 'QueryEntityPill';

export default QueryEntityPill;
